package com.kexun.service.atomic;

import com.alibaba.fastjson.JSONObject;
import com.kexun.constant.BaseConsts;
import com.kexun.entity.FieldDefineEntity;
import com.kexun.entity.FieldParamEntity;
import com.kexun.factory.HandleXmlFactory;
import com.kexun.factory.HandleXmlFactoryBean;
import com.kexun.model.xml.*;
import com.kexun.model.xml.trs.*;
import com.kexun.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * Description: 衍生字段的算法
 *
 * @author lgs
 * @date 2023/11/21
 */
@Service
@Slf4j
public class DeriveFieldAtomicService {

    @Autowired
    HandleXmlFactory handleXmlFactory;

    /**
     * 基本信息-手机号码信息段
     *
     * @param fieldParamEntities 字段参数列表
     * @param fieldDefineEntity  字段定义表信息
     * @param reportXmlBase      报文数据
     * @return
     */
    public Object baseInfoOfPhoneCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        /**
         * 计算思路
         * 1、循环手机号码信息段
         * 2、获取到当前的日期和手机号码信息段里的更新日期进行比较，分别算出来3、6、12、24月的手机号码数
         *
         */
        long count = 0L;
        if (reportXmlBase instanceof BasePhone) {
            BasePhone basePhone = (BasePhone) reportXmlBase;
            List<BasePhoneInf> basePhoneInfList = basePhone.getBasePhoneInfList();
            //月份参数
            String param = fieldDefineEntity.getParam();
            if (!StringUtils.isEmpty(param) && param.startsWith(BaseConsts.PARAM_SIGN_AT) && param.endsWith(BaseConsts.PARAM_SIGN_MONTH)) {
                param = param.substring(1, param.indexOf(BaseConsts.PARAM_SIGN_MONTH));
                //根据当前日期和月份参数，倒退到几个月之前的月份
                Date curretDate = new Date();
                Date beginDate = DateUtils.lastMonthDate(curretDate, Integer.valueOf(param), BaseConsts.PARAM_SIGN_MONTH);
                count = basePhoneInfList.stream()
                        .filter(
                                (e) -> (e.getUpdateDateInfo() != null
                                        && DateUtils.convertStrToDate(e.getUpdateDateInfo(), DateUtils.YYYY_MM_DD).compareTo(beginDate) > 0
                                        && DateUtils.convertStrToDate(e.getUpdateDateInfo(), DateUtils.YYYY_MM_DD).compareTo(curretDate) < 0))
                        .count();
            } else {
                count = basePhoneInfList.stream().count();

            }
        }
        return count;
    }

    /**
     * 统计住宅个数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object residenceCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        long count = 0L;
        if (reportXmlBase instanceof BaseResidence) {
            BaseResidence baseResidence = (BaseResidence) reportXmlBase;
            List<BaseResidenceInf> baseResidenceInfs = baseResidence.getBaseResidenceInfs();
            String param = fieldDefineEntity.getParam();
            if (!StringUtils.isEmpty(param) && param.startsWith(BaseConsts.PARAM_SIGN_AT) && param.endsWith(BaseConsts.PARAM_SIGN_YEAR)) {
                param = param.substring(1, param.indexOf(BaseConsts.PARAM_SIGN_YEAR));
                //根据当前日期和月份参数，计算出几年之前的月份
                Date curretDate = new Date();
                Date beginDate = DateUtils.lastMonthDate(curretDate, Integer.valueOf(param), BaseConsts.PARAM_SIGN_YEAR);
                count = baseResidenceInfs.stream().filter((e) -> e.getUpdateDateInfo() != null
                        && DateUtils.convertStrToDate(e.getUpdateDateInfo(), DateUtils.YYYY_MM_DD).compareTo(beginDate) > 0
                        && DateUtils.convertStrToDate(e.getUpdateDateInfo(), DateUtils.YYYY_MM_DD).compareTo(curretDate) < 0
                ).count();
            } else {
                count = baseResidenceInfs.stream().count();
            }

        }
        return count;
    }

    /**
     * 是否包含自有住房
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object isExesitOwnHouse(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        Boolean flag = false;
        if (reportXmlBase instanceof BaseResidence) {
            BaseResidence baseResidence = (BaseResidence) reportXmlBase;
            List<BaseResidenceInf> baseResidenceInfs = baseResidence.getBaseResidenceInfs();
            String param = fieldDefineEntity.getParam();
            if (!StringUtils.isEmpty(param) && param.startsWith(BaseConsts.PARAM_SIGN_CONTAIN)) {
                String paramValue = param.substring(param.indexOf(BaseConsts.PARAM_SIGN_CONTAIN), param.length());
                flag = baseResidenceInfs.stream()
                        .anyMatch((e) -> e.getResiStatusDesc().contains(paramValue));
            }
        }
        return flag;
    }

    /**
     * 统计工作单位数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object workUnitCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        Long count = 0L;
        if (reportXmlBase instanceof BaseVocation) {
            BaseVocation baseVocation = (BaseVocation) reportXmlBase;
            List<BaseVocationInf> baseVocationInfList = baseVocation.getBaseVocationInfList();
            String param = fieldDefineEntity.getParam();
            if (!StringUtils.isEmpty(param) && param.startsWith(BaseConsts.PARAM_SIGN_AT) && param.endsWith(BaseConsts.PARAM_SIGN_YEAR)) {
                param = param.substring(1, param.indexOf(BaseConsts.PARAM_SIGN_YEAR));
                //根据当前日期和月份参数，计算出几年之前的月份
                Date curretDate = new Date();
                Integer curYear = DateUtils.getUnitDate(curretDate, DateUtils.UNIT_YEARS);
                Integer beginYear = curYear - Integer.valueOf(param);
                count = baseVocationInfList.stream().filter((e) -> e.getBeginYear() != null
                        && Integer.valueOf(e.getBeginYear()).compareTo(beginYear) >= 0
                ).count();

            } else {
                count = baseVocationInfList.stream().count();
            }
        }
        return count;
    }

    /**
     * 最早/新进入本单位年份距报告日期时长
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object workUnitLastCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        Long count = 0L;
        return count;
    }

    /**
     * 是否存在担保方式为抵押的贷款
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object isExistMotgaLoanCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Boolean result = false;
        AtomicReference<Boolean> loopAcctMotgaLoan = new AtomicReference<>(false);
        AtomicReference<Boolean> loopQuotaAcctMotgaLoan = new AtomicReference<>(false);
        AtomicReference<Boolean> noLoopAcctMotgaLoan = new AtomicReference<>(false);
        fieldParamEntities.stream().forEach((e) -> {
            if (e.getParamType().equals("xml")) {
                String paramValue = e.getParamValue();
                HandleXmlFactoryBean factoryBean = handleXmlFactory.getFactoryBean(paramValue);
                Class<? extends IXmlBase> xmlModelBean = factoryBean.getModelBean();
                IXmlBase IXmlBase = null;
                try {
                    IXmlBase = xmlModelBean.newInstance();
                } catch (InstantiationException instantiationException) {
                    instantiationException.printStackTrace();
                } catch (IllegalAccessException illegalAccessException) {
                    illegalAccessException.printStackTrace();
                }
                if (jsonObject.get(paramValue) != null) {
                    JSONObject jsonObject1 = (JSONObject) jsonObject.get(paramValue);
                    String json = jsonObject1.toJSONString();
                    //json转化成JavaBean
                    IXmlBase jsonBean = JSONObject.parseObject(json, IXmlBase.getClass());
                    //循环额度下分账户
                    if (jsonBean instanceof LoopAcctTrsInf) {
                        LoopAcctTrsInf loopAcctTrsInf = (LoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopAcctTrsInf.getTrsAcctInfList();
                        loopAcctMotgaLoan.set(trsAcctInfList.stream()
                                .anyMatch((trsAcctInf) -> trsAcctInf.getTrsBase().getGuarModeDesc().contains(e.getParamExtend())));
                    }
                    //循环额度下分账户
                    if (jsonBean instanceof LoopQuotaAcctTrsInf) {
                        LoopQuotaAcctTrsInf loopQuotaAcctTrsInf = (LoopQuotaAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopQuotaAcctTrsInf.getTrsAcctInfList();
                        loopQuotaAcctMotgaLoan.set(trsAcctInfList.stream()
                                .anyMatch((trsAcctInf) -> trsAcctInf.getTrsBase().getGuarModeDesc().contains(e.getParamExtend())));
                    }
                    //非循环贷账户
                    if (jsonBean instanceof NoLoopAcctTrsInf) {
                        NoLoopAcctTrsInf noLoopAcctTrsInf = (NoLoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = noLoopAcctTrsInf.getTrsAcctInfList();
                        noLoopAcctMotgaLoan.set(trsAcctInfList.stream()
                                .anyMatch((trsAcctInf) -> trsAcctInf.getTrsBase().getGuarModeDesc().contains(e.getParamExtend())));
                    }
                } else {
                    log.info("方法【是否存在担保方式为抵押的贷款】获取节点{}为空", paramValue);
                }
            }
        });
        if (loopAcctMotgaLoan.get() || loopQuotaAcctMotgaLoan.get() || noLoopAcctMotgaLoan.get()) {
            result = true;
        }
        return result;
    }

    /**
     * 担保方式为信用的贷款数量
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object creditLoanCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<Integer> sum = new AtomicReference<>(0);
        fieldParamEntities.stream().forEach((e) -> {
            if (e.getParamType().equals("xml")) {
                String paramValue = e.getParamValue();
                HandleXmlFactoryBean factoryBean = handleXmlFactory.getFactoryBean(paramValue);
                Class<? extends IXmlBase> xmlModelBean = factoryBean.getModelBean();
                IXmlBase IXmlBase = null;
                try {
                    IXmlBase = xmlModelBean.newInstance();
                } catch (InstantiationException instantiationException) {
                    instantiationException.printStackTrace();
                } catch (IllegalAccessException illegalAccessException) {
                    illegalAccessException.printStackTrace();
                }
                if (jsonObject.get(paramValue) != null) {
                    JSONObject jsonObject1 = (JSONObject) jsonObject.get(paramValue);
                    String json = jsonObject1.toJSONString();
                    //json转化成JavaBean
                    IXmlBase jsonBean = JSONObject.parseObject(json, IXmlBase.getClass());
                    //循环额度下分账户
                    if (jsonBean instanceof LoopAcctTrsInf) {
                        LoopAcctTrsInf loopAcctTrsInf = (LoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopAcctTrsInf.getTrsAcctInfList();
                        Long count = trsAcctInfList.stream()
                                .filter((trsAcctInf) -> trsAcctInf.getTrsBase().getGuarModeDesc().contains(e.getParamExtend()))
                                .count();
                        sum.set(sum.get() + count.intValue());
                    }
                    //循环额度下分账户
                    if (jsonBean instanceof LoopQuotaAcctTrsInf) {
                        LoopQuotaAcctTrsInf loopQuotaAcctTrsInf = (LoopQuotaAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopQuotaAcctTrsInf.getTrsAcctInfList();
                        Long count = trsAcctInfList.stream()
                                .filter((trsAcctInf) -> trsAcctInf.getTrsBase().getGuarModeDesc().contains(e.getParamExtend()))
                                .count();
                        sum.set(sum.get() + count.intValue());
                    }
                    //非循环贷账户
                    if (jsonBean instanceof NoLoopAcctTrsInf) {
                        NoLoopAcctTrsInf noLoopAcctTrsInf = (NoLoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = noLoopAcctTrsInf.getTrsAcctInfList();
                        Long count = trsAcctInfList.stream()
                                .filter((trsAcctInf) -> trsAcctInf.getTrsBase().getGuarModeDesc().contains(e.getParamExtend()))
                                .count();
                        sum.set(sum.get() + count.intValue());
                    }
                } else {
                    log.info("方法【是否存在担保方式为抵押的贷款】获取节点{}为空", paramValue);
                }
            }

        });
        return sum.get();
    }

    /**
     * 该用户所有逾期账户最长逾期/透支月数最大值
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object maxOverMonthCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        Integer result = 0;
        if (reportXmlBase instanceof Overdraft) {
            Overdraft overdraft = (Overdraft) reportXmlBase;
            List<OverdraftInfo> overdraftInfoList = overdraft.getOverdraftInfoList();
            result = overdraftInfoList.stream()
                    .filter(e -> e.getOverMonth() != null)
                    .collect(Collectors.maxBy((e1, e2) -> Integer.compare(e1.getOverMonth(), e2.getOverMonth())))
                    .map(OverdraftInfo::getOverMonth)
                    .orElse(0);
        }
        return result;
    }

    /**
     * 该用户所有逾期账户数加总
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object overAcctCount(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        Integer result = 0;
        if (reportXmlBase instanceof Overdraft) {
            Overdraft overdraft = (Overdraft) reportXmlBase;
            List<OverdraftInfo> overdraftInfoList = overdraft.getOverdraftInfoList();
            result = overdraftInfoList.stream()
                    .filter(e -> e.getAccSum() != null)
                    .collect(Collectors.summingInt(OverdraftInfo::getAccSum));
        }
        return result;
    }

    /**
     * 相关还款责任总账户数（担保+其他+个人+企业）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object relRepayAcctCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        Integer count = 0;
        if (reportXmlBase instanceof SmryRepay) {
            SmryRepay smryRepay = (SmryRepay) reportXmlBase;
            List<SmryRepayInfo> smryRepayInfoList = smryRepay.getSmryRepayInfoList();
            count = smryRepayInfoList.stream()
                    .filter((e) -> e.getAccSum() != null)
                    .collect(Collectors.summingInt(SmryRepayInfo::getAccSum));

        }
        return count;
    }

    /**
     * 相关还款责任总担保金额+总还款责任金额（个人+企业）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object relRepayAmtSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        BigDecimal sum = new BigDecimal("0.00");
        if (reportXmlBase instanceof SmryRepay) {
            SmryRepay smryRepay = (SmryRepay) reportXmlBase;
            List<SmryRepayInfo> smryRepayInfoList = smryRepay.getSmryRepayInfoList();
            sum = smryRepayInfoList.stream()
                    .filter((e) -> e.getRelPayRespSum() != null)
                    .map(SmryRepayInfo::getRelPayRespSum)
                    .reduce(BigDecimal.ZERO, BigDecimal::add)
                    .setScale(2, RoundingMode.HALF_UP);
        }
        return sum;
    }

    /**
     * 相关还款责任账户总担保余额+总其他余额（个人+企业）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object relRepayBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        BigDecimal sum = new BigDecimal("0.00");
        if (reportXmlBase instanceof SmryRepay) {
            SmryRepay smryRepay = (SmryRepay) reportXmlBase;
            List<SmryRepayInfo> smryRepayInfoList = smryRepay.getSmryRepayInfoList();
            sum = smryRepayInfoList.stream()
                    .filter((e) -> e.getBalance() != null)
                    .map(SmryRepayInfo::getBalance)
                    .reduce(BigDecimal.ZERO, BigDecimal::add)
                    .setScale(2, RoundingMode.HALF_UP);
        }
        return sum;
    }

    /**
     * 相关还款责任账户总担保余额+总其他余额（个人+企业）/相关还款责任账户总担保金额+总其他金额（个人+企业）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object relRepayBalAmtPercCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        BigDecimal result = new BigDecimal("0.00");
        if (reportXmlBase instanceof SmryRepay) {
            SmryRepay smryRepay = (SmryRepay) reportXmlBase;
            List<SmryRepayInfo> smryRepayInfoList = smryRepay.getSmryRepayInfoList();
            BigDecimal balSum = smryRepayInfoList.stream()
                    .filter((e) -> e.getBalance() != null)
                    .map(SmryRepayInfo::getBalance)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);

            BigDecimal sum = smryRepayInfoList.stream()
                    .filter((e) -> e.getRelPayRespSum() != null)
                    .map(SmryRepayInfo::getRelPayRespSum)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            try {
                result = balSum.divide(sum, 2, BigDecimal.ROUND_CEILING);
            } catch (ArithmeticException e) {
            }

        }
        return result;
    }

    /**
     * 相关还款责任总担保余额-个人/相关还款责任总担保金额-个人
     * 相关还款责任总担保余额-企业/相关还款责任总担保金额-企业
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object relRepayBalAmtTypePercCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        BigDecimal result = new BigDecimal("0.00");
        if (reportXmlBase instanceof SmryRepay) {
            SmryRepay smryRepay = (SmryRepay) reportXmlBase;
            //1为个人 2-为企业
            String param = fieldDefineEntity.getParam();
            List<SmryRepayInfo> smryRepayInfoList = smryRepay.getSmryRepayInfoList();
            BigDecimal sum = smryRepayInfoList.stream()
                    .filter((e) -> e.getBoTypeCode() != null && e.getBoTypeCode().equals(param))
                    .map(SmryRepayInfo::getRelPayRespSum)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);


            BigDecimal bal = smryRepayInfoList.stream()
                    .filter((e) -> e.getBoTypeCode() != null && e.getBoTypeCode().equals(param))
                    .map(SmryRepayInfo::getBalance)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            try {
                result = sum.divide(bal, 2, BigDecimal.ROUND_CEILING);
            } catch (ArithmeticException e) {

            }
        }
        return result;
    }

    /**
     * 相关还款责任总担保余额-担保责任/相关还款责任总担保金额-担保责任
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param reportXmlBase
     * @return
     */
    public Object relRepayBalGuaPercCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, IXmlBase reportXmlBase) {
        BigDecimal result = new BigDecimal("0.00");
        if (reportXmlBase instanceof SmryRepay) {
            SmryRepay smryRepay = (SmryRepay) reportXmlBase;
            //1-担保责任  2-其它
            String param = fieldDefineEntity.getParam();
            List<SmryRepayInfo> smryRepayInfoList = smryRepay.getSmryRepayInfoList();
            BigDecimal sum = smryRepayInfoList.stream()
                    .filter((e) -> e.getRelPayRespCode() != null && e.getRelPayRespCode().equals(param))
                    .map(SmryRepayInfo::getRelPayRespSum)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);

            BigDecimal bal = smryRepayInfoList.stream()
                    .filter((e) -> e.getRelPayRespCode() != null && e.getRelPayRespCode().equals(param))
                    .map(SmryRepayInfo::getBalance)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            try {
                result = sum.divide(bal, 2, BigDecimal.ROUND_CEILING);
            } catch (ArithmeticException e) {

            }

        }
        return result;
    }

    /**
     * 根据jsonName获取转化成对应的JavaBean实例
     *
     * @param jsonName
     * @param jsonObject
     * @return
     */
    private IXmlBase getXmlBase(String jsonName, JSONObject jsonObject) {
        HandleXmlFactoryBean factoryBean = handleXmlFactory.getFactoryBean(jsonName);
        Class<? extends IXmlBase> xmlModelBean = factoryBean.getModelBean();
        IXmlBase xmlBase = null;
        try {
            xmlBase = xmlModelBean.newInstance();
        } catch (InstantiationException instantiationException) {
            instantiationException.printStackTrace();
        } catch (IllegalAccessException illegalAccessException) {
            illegalAccessException.printStackTrace();
        }
        if (jsonObject.get(jsonName) != null) {
            JSONObject jsonObject1 = (JSONObject) jsonObject.get(jsonName);
            String json = jsonObject1.toJSONString();
            //json转化成JavaBean
            xmlBase = JSONObject.parseObject(json, xmlBase.getClass());
        } else {
            log.error("获取不到节点为{}，这个的数据", jsonName);
        }
        return xmlBase;
    }

    /**
     * 当前一共逾期期数
     * 统计循环贷账户、循环额度下分账户、非循环贷账户信息段各个账户的逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object currOverPrdCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<Integer> sum = new AtomicReference<>(0);
        fieldParamEntities.stream().forEach((e) -> {
            if (e.getParamType().equals("xml")) {
                String paramValue = e.getParamValue();
                IXmlBase jsonBean = getXmlBase(paramValue, jsonObject);
                if (jsonBean != null) {
                    //循环贷账户
                    if (jsonBean instanceof LoopAcctTrsInf) {
                        LoopAcctTrsInf loopAcctTrsInf = (LoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopAcctTrsInf.getTrsAcctInfList();
                        if (trsAcctInfList != null) {
                            Integer loopAcctOverdPrdSum = trsAcctInfList.stream()
                                    .filter((trsAcctInf) -> trsAcctInf.getTrsLateMonth().getOverdPrd() != null)
                                    .collect(Collectors.summingInt((trsAcctInf) -> trsAcctInf.getTrsLateMonth().getOverdPrd()));
                            sum.set(sum.get() + loopAcctOverdPrdSum);
                        }
                    }
                    //循环额度下分账户
                    if (jsonBean instanceof LoopQuotaAcctTrsInf) {
                        LoopQuotaAcctTrsInf loopQuotaAcctTrsInf = (LoopQuotaAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopQuotaAcctTrsInf.getTrsAcctInfList();
                        if (trsAcctInfList != null) {
                            Integer loopQuotaAcctOverdPrdSum = trsAcctInfList.stream()
                                    .filter((trsAcctInf) -> trsAcctInf.getTrsLateMonth().getOverdPrd() != null)
                                    .collect(Collectors.summingInt((trsAcctInf) -> trsAcctInf.getTrsLateMonth().getOverdPrd()));
                            sum.set(sum.get() + loopQuotaAcctOverdPrdSum.intValue());
                        }
                    }
                    //非循环贷账户
                    if (jsonBean instanceof NoLoopAcctTrsInf) {
                        NoLoopAcctTrsInf noLoopAcctTrsInf = (NoLoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = noLoopAcctTrsInf.getTrsAcctInfList();
                        if (trsAcctInfList != null) {
                            Integer noLoopAcctOverdPrdSum = trsAcctInfList.stream()
                                    .filter((trsAcctInf) -> trsAcctInf.getTrsLateMonth().getOverdPrd() != null)
                                    .collect(Collectors.summingInt((trsAcctInf) -> trsAcctInf.getTrsLateMonth().getOverdPrd()));
                            sum.set(sum.get() + noLoopAcctOverdPrdSum.intValue());
                        }
                    }
                }
            }
        });
        return sum.get();
    }

    /**
     * 贷款信息-逾期31-60天未还本金（合计）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object over31_60PrinSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<BigDecimal> currOverPrdAmtSum = new AtomicReference<>(new BigDecimal("0.00"));
        commonCalculate(fieldParamEntities, jsonObject, currOverPrdAmtSum);
        return currOverPrdAmtSum.get();
    }

    /**
     * 贷款信息-逾期61-90天未还本金（合计）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object over61_90PrinSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<BigDecimal> currOverPrdAmtSum = new AtomicReference<>(new BigDecimal("0.00"));
        commonCalculate(fieldParamEntities, jsonObject, currOverPrdAmtSum);
        return currOverPrdAmtSum.get();
    }

    /**
     * 贷款信息-逾期61-90天未还本金（合计）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object over91_180PrinSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<BigDecimal> currOverPrdAmtSum = new AtomicReference<>(new BigDecimal("0.00"));
        commonCalculate(fieldParamEntities, jsonObject, currOverPrdAmtSum);
        return currOverPrdAmtSum.get();
    }

    /**
     * 贷款信息-逾期180天未还本金（合计）
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object over180PrinSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<BigDecimal> currOverPrdAmtSum = new AtomicReference<>(new BigDecimal("0.00"));
        commonCalculate(fieldParamEntities, jsonObject, currOverPrdAmtSum);
        return currOverPrdAmtSum.get();
    }

    /**
     * 当前一共逾期总额
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object currOverPrdAmtSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        AtomicReference<BigDecimal> currOverPrdAmtSum = new AtomicReference<>(new BigDecimal("0.00"));
        commonCalculate(fieldParamEntities, jsonObject, currOverPrdAmtSum);
        return currOverPrdAmtSum.get();
    }

    /**
     * 贷款信息-最近一笔结清贷款的结清距今月数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lastSettleLoanSettleMonthCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Integer spaceByUnit = 0;
        TrsAcctInf lastTrsAcctInf = getSummaryTrsAcctInf(fieldParamEntities, jsonObject);
        //开户日期
        if (lastTrsAcctInf != null) {
            Date openDate = DateUtils.convertStrToDate(lastTrsAcctInf.getTrsLate().getSettleDate(), DateUtils.YYYY_MM_DD);
            //当前日期
            Date currDate = new Date();
            spaceByUnit = DateUtils.getSpaceByUnit(openDate, currDate, DateUtils.UNIT_MOUTHS);
        }

        return spaceByUnit;
    }

    /**
     * 贷款信息-最近一笔结清贷款的发放距今月数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lastSettleLoanStartMonthCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Integer spaceByUnit = 0;
        TrsAcctInf lastTrsAcctInf = getSummaryTrsAcctInf(fieldParamEntities, jsonObject);
        if (lastTrsAcctInf != null) {
            //开户日期
            Date openDate = DateUtils.convertStrToDate(lastTrsAcctInf.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD);
            //当前日期
            Date currDate = new Date();
            spaceByUnit = DateUtils.getSpaceByUnit(openDate, currDate, DateUtils.UNIT_MOUTHS);
        }
        return spaceByUnit;
    }

    /**
     * 贷款信息-最近一笔结清贷款的贷款金额
     * 计算思路:循环贷账户、循环额度下分账户、非循环贷账户 分别进行循环，
     * 然后获取到结清状态的数据，并比较开户日期和当前日期进行比较、找到最近的一笔记录
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lastSettleLoanAmtCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        TrsAcctInf lastTrsAcctInf = getSummaryTrsAcctInf(fieldParamEntities, jsonObject);
        return lastTrsAcctInf.getTrsBase().getLoanAmt();
    }

    /**
     * 获取循环贷账户、循环额度下分账户、非循环贷账户三个信息段的最近结清的贷款信息
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @return
     */
    private TrsAcctInf getSummaryTrsAcctInf(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject) {
        Map<String, List<FieldParamEntity>> mapFieldParam = fieldParamEntities.stream()
                .filter((e) -> e.getParamValue() != null && !e.getParamValue().equals(""))
                .collect(Collectors.groupingBy((e) -> e.getParamValue()));
        List<TrsAcctInf> acctTrsInfs = new ArrayList<>();
        mapFieldParam.forEach((k, v) -> {
            IXmlBase jsonBean = getXmlBase(k, jsonObject);
            if (jsonBean != null) {
                //循环贷账户
                if (jsonBean instanceof LoopAcctTrsInf) {
                    LoopAcctTrsInf loopAcctTrsInf = (LoopAcctTrsInf) jsonBean;
                    List<TrsAcctInf> trsAcctInfList = loopAcctTrsInf.getTrsAcctInfList();
                    if (trsAcctInfList != null) {
                        TrsAcctInf lastTrsAcctInf = getLastTrsAcctInf(trsAcctInfList);
                        BigDecimal acctCredLine = lastTrsAcctInf.getTrsBase().getAcctCredLine();
                        lastTrsAcctInf.getTrsBase().setLoanAmt(acctCredLine);
                        acctTrsInfs.add(lastTrsAcctInf);
                    }
                }
                //循环额度下分账户
                if (jsonBean instanceof LoopQuotaAcctTrsInf) {
                    LoopQuotaAcctTrsInf loopQuotaAcctTrsInf = (LoopQuotaAcctTrsInf) jsonBean;
                    List<TrsAcctInf> trsAcctInfList = loopQuotaAcctTrsInf.getTrsAcctInfList();
                    if (trsAcctInfList != null) {
                        TrsAcctInf lastTrsAcctInf = getLastTrsAcctInf(trsAcctInfList);
                        acctTrsInfs.add(lastTrsAcctInf);
                    }
                }
                //非循环贷账户
                if (jsonBean instanceof NoLoopAcctTrsInf) {
                    NoLoopAcctTrsInf noLoopAcctTrsInf = (NoLoopAcctTrsInf) jsonBean;
                    List<TrsAcctInf> trsAcctInfList = noLoopAcctTrsInf.getTrsAcctInfList();
                    if (trsAcctInfList != null) {
                        TrsAcctInf lastTrsAcctInf = getLastTrsAcctInf(trsAcctInfList);
                        acctTrsInfs.add(lastTrsAcctInf);
                    }
                }
            }

        });
        TrsAcctInf lastTrsAcctInf = getLastTrsAcctInf(acctTrsInfs);
        return lastTrsAcctInf;
    }

    /**
     * 获取最近一笔结清贷款
     * 当贷款是结清状态时候，是不存在最近一次月度表现信息段，只展示最新表现信息
     *
     * @param trsAcctInfList
     * @return
     */
    private TrsAcctInf getLastTrsAcctInf(List<TrsAcctInf> trsAcctInfList) {

        //获取到最近结清的一笔
        TrsAcctInf trsAcctInf = trsAcctInfList.stream()
                .filter((e) -> e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_3)
                        && e.getTrsLate().getSettleDate() != null)
                .sorted((new Comparator<TrsAcctInf>() {
                    @Override
                    public int compare(TrsAcctInf o1, TrsAcctInf o2) {
                        Date d1 = DateUtils.convertStrToDate(o1.getTrsLate().getSettleDate(), DateUtils.YYYY_MM_DD);
                        Date d2 = DateUtils.convertStrToDate(o2.getTrsLate().getSettleDate(), DateUtils.YYYY_MM_DD);
                        //正序
                        return d1.compareTo(d2);
                    }
                }))
                .findFirst()
                .orElse(null);

        return trsAcctInf;
    }

    /**
     * 贷款信息-历史贷款总法人机构数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object hisLoanLegalPerOrgCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        long count = trsAcctInfs.stream()
                .map((e) -> e.getTrsBase().getBusiOrgCode())
                .distinct()
                .count();
        return count;
    }

    /**
     * 贷款信息-当前同时在贷的贷款机构数
     * 统计的是在贷的贷款机构数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curInloanOrgCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        long count = trsAcctInfs.stream()
                .filter((e) -> (e.getTrsLateMonth() != null && !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                        || (e.getTrsLate() != null && !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_3))
                ).count();
        return count;
    }

    /**
     * 贷款信息-是否有非银行贷款客户
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object isExistBankLoanCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        long count = trsAcctInfs.stream()
                .filter((e) -> e.getTrsBase().getMagOrgTypeDesc().contains("银行"))
                .count();
        return count;
    }

    /**
     * 贷款信息-当前贷款LTV
     * 循环贷账户、循环额度下分账户、非循环贷账户信息段的余额之和和借款金额之和的比值
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        BigDecimal result = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> resultTrsAcctInfs = trsAcctInfs.stream()
                .filter((e) -> (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                        || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .collect(Collectors.toList());
        //贷款金额汇总
        BigDecimal loanAmtSum = resultTrsAcctInfs.stream()
                .map((e) -> {
                    return Optional.ofNullable(e.getTrsBase().getLoanAmt()).orElse(new BigDecimal("0.00"));
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        //余额汇总
        BigDecimal balSum = resultTrsAcctInfs.stream().map((e) -> {
            BigDecimal bigDecimal = new BigDecimal("0.00");
            if (e.getTrsLateMonth() != null) {
                bigDecimal = Optional.ofNullable(e.getTrsLateMonth().getBusiBal()).orElse(new BigDecimal("0.00"));
            }
            return bigDecimal;
        }).reduce(BigDecimal.ZERO, BigDecimal::add);

        result = balSum.divide(loanAmtSum, 2, BigDecimal.ROUND_CEILING);
        return result;
    }

    /**
     * 贷款信息-当前贷款最高LTV
     * 思路：循环每一个账户，过滤掉结清的和呆账的，计算每一个账户的LTV，然后找到最大的LTV最大的
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanHightLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> resultTrsAcctInfs = trsAcctInfs.stream()
                .filter((e) ->
                        (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                                || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .collect(Collectors.toList());
        BigDecimal hightLTV = new BigDecimal("0.00");
        for (TrsAcctInf resultTrsAcctInf : resultTrsAcctInfs) {
            //贷款金额
            BigDecimal loanAmt = Optional.ofNullable(resultTrsAcctInf.getTrsBase().getLoanAmt()).orElse(new BigDecimal("0.00"));
            BigDecimal balance = new BigDecimal("0.00");
            //贷款余额
            if (resultTrsAcctInf.getTrsLate() != null) {
                balance = Optional.ofNullable(resultTrsAcctInf.getTrsLate().getBalance()).orElse(new BigDecimal("0.00"));
            } else {
                balance = Optional.ofNullable(resultTrsAcctInf.getTrsLateMonth().getBusiBal()).orElse(new BigDecimal("0.00"));
            }
            BigDecimal result = balance.divide(loanAmt, 2, BigDecimal.ROUND_CEILING);
            if (result.compareTo(hightLTV) > 0) {
                hightLTV = result;
            }
        }
        return hightLTV;
    }

    /**
     * 贷款信息-当前贷款最低LTV
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLowestLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> resultTrsAcctInfs = trsAcctInfs.stream()
                .filter((e) ->
                        (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                                || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .collect(Collectors.toList());
        BigDecimal lowestLTV = new BigDecimal("0.00");
        for (TrsAcctInf resultTrsAcctInf : resultTrsAcctInfs) {
            //贷款金额
            BigDecimal loanAmt = Optional.ofNullable(resultTrsAcctInf.getTrsBase().getLoanAmt()).orElse(new BigDecimal("0.00"));
            BigDecimal balance = new BigDecimal("0.00");
            //贷款余额
            if (resultTrsAcctInf.getTrsLate() != null) {
                balance = Optional.ofNullable(resultTrsAcctInf.getTrsLate().getBalance()).orElse(new BigDecimal("0.00"));
            } else {
                balance = Optional.ofNullable(resultTrsAcctInf.getTrsLateMonth().getBusiBal()).orElse(new BigDecimal("0.00"));
            }
            BigDecimal result = balance.divide(loanAmt, 2, BigDecimal.ROUND_CEILING);
            if (result.compareTo(lowestLTV) < 0) {
                lowestLTV = result;
            }
        }
        return lowestLTV;
    }

    /**
     * 当前贷款平均LTV
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curAvgLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> resultTrsAcctInfs = trsAcctInfs.stream()
                .filter((e) ->
                        (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                                || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .collect(Collectors.toList());
        BigDecimal curSumLTV = new BigDecimal("0.00");
        int countLtv = 0;
        for (TrsAcctInf resultTrsAcctInf : resultTrsAcctInfs) {
            //贷款金额
            BigDecimal loanAmt = Optional.ofNullable(resultTrsAcctInf.getTrsBase().getLoanAmt()).orElse(new BigDecimal("0.00"));
            BigDecimal balance = new BigDecimal("0.00");
            //贷款余额
            if (resultTrsAcctInf.getTrsLate() != null) {
                balance = Optional.ofNullable(resultTrsAcctInf.getTrsLate().getBalance()).orElse(new BigDecimal("0.00"));
            } else if (resultTrsAcctInf.getTrsLateMonth() != null) {
                balance = Optional.ofNullable(resultTrsAcctInf.getTrsLateMonth().getBusiBal()).orElse(new BigDecimal("0.00"));
            }
            BigDecimal result = balance.divide(loanAmt, 2, BigDecimal.ROUND_CEILING);
            if (result.compareTo(BigDecimal.ZERO) > 0) {
                curSumLTV.add(curSumLTV);
                countLtv++;
            }
        }
        BigDecimal curAvgLTV = curSumLTV.divide(new BigDecimal(countLtv), 2, BigDecimal.ROUND_CEILING);
        return curAvgLTV;
    }

    /**
     * 贷款信息-当前房贷LTV
     * 业务种类为：11-个人住房商业贷款，12-个人商用房（含商住两用）贷款，13-个人住房公积金贷款
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curHouseLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonTypeLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("11", "12", "13"));
    }

    /**
     * 贷款信息-当前车贷LTV
     * 业务种类为：21-个人汽车消费贷款
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curCarLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonTypeLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("21"));
    }

    /**
     * 贷款信息-当前经营贷LTV
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curBusLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonTypeLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("41"));
    }

    /**
     * 贷款信息-当前消费贷LTV
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curConsLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonTypeLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("91"));
    }

    /**
     * 通用的贷款类型的LTV计算方法
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @param loanTypeCodeList
     * @return
     */
    private BigDecimal curCommonTypeLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject, List loanTypeCodeList) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> trsAcctInfList = trsAcctInfs.stream()
                .filter((e) ->
                        (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                                || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .filter((e) -> {
                            if (loanTypeCodeList.contains(e.getTrsBase().getLoanTypeCode())) {
                                return true;
                            } else {
                                return false;
                            }
                        }
                )
                .collect(Collectors.toList());
        //贷款金额
        BigDecimal loanAmtSum = trsAcctInfList.stream()
                .map((e) -> Optional.ofNullable(e.getTrsBase().getLoanAmt()).orElse(new BigDecimal("0.00")))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        //贷款余额
        BigDecimal balAmtSum = trsAcctInfList.stream()
                .map((e) -> {
                            if (e.getTrsLateMonth() != null) {
                                return Optional.ofNullable(e.getTrsLateMonth().getBusiBal()).orElse(new BigDecimal("0.00"));
                            } else {
                                return new BigDecimal("0.00");
                            }
                        }
                )
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal commonTV = new BigDecimal("0.00");
        if (balAmtSum.compareTo(BigDecimal.ZERO) > 0) {
            commonTV = balAmtSum.divide(loanAmtSum, 2, BigDecimal.ROUND_CEILING);
        }
        return commonTV;
    }

    /**
     * 贷款信息-当前银行贷LTV
     * 贷款机构类型为商业银行、外资银行、村镇银行、住房储蓄银行的贷款记录为银行贷。
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public BigDecimal curBankLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonMagOrgLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("商业银行", "外资银行", "村镇银行", "住房储蓄银行"));
    }

    /**
     * 贷款信息-当前消金贷LTV
     * 检索管理机构为“消费金融公司”、“汽车金融公司”、“信托公司'
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public BigDecimal curConsMoneyLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonMagOrgLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("消费金融公司", "汽车金融公司", "信托公司"));
    }

    /**
     * 贷款信息-当前小贷LTV
     * 贷款机构类型为机构、小额信贷公司、财务公司的贷款记录为小贷公司贷款。
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public BigDecimal curSamlLoanComLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return curCommonMagOrgLTVCalculate(fieldParamEntities, fieldDefineEntity, jsonObject, Arrays.asList("小额信贷公司", "财务公司"));
    }

    /**
     * 通用的业务管理机构的LTV计算方法
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @param magOrgCodeList
     * @return
     */
    private BigDecimal curCommonMagOrgLTVCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject, List<String> magOrgCodeList) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> trsAcctInfList = trsAcctInfs.stream()
                .filter((e) ->
                        (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                                || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .filter((e) -> isContain(magOrgCodeList, e.getTrsBase().getMagOrgTypeDesc()))
                .collect(Collectors.toList());
        //贷款金额
        BigDecimal loanAmtSum = trsAcctInfList.stream()
                .map((e) -> Optional.ofNullable(e.getTrsBase().getLoanAmt()).orElse(new BigDecimal("0.00")))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        //贷款余额
        BigDecimal balAmtSum = trsAcctInfList.stream()
                .map(
                        (e) -> {
                            if (e.getTrsLateMonth() != null) {
                                return Optional.ofNullable(e.getTrsLateMonth().getBusiBal()).orElse(new BigDecimal("0.00"));
                            } else {
                                return new BigDecimal("0.00");
                            }
                        }
                )
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal commonTV = new BigDecimal("0.00");
        if (balAmtSum.compareTo(BigDecimal.ZERO) > 0) {
            commonTV = balAmtSum.divide(loanAmtSum, 2, BigDecimal.ROUND_CEILING);
        }

        return commonTV;
    }

    /**
     * 贷款信息-当前贷款最大逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curMaxOverdPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Integer overdPrd = 0;
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        TrsAcctInf trsAcctInf = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getOverdPrd() != null)
                .collect(Collectors.maxBy((e1, e2) -> Integer.compare(e1.getTrsLateMonth().getOverdPrd(), e2.getTrsLateMonth().getOverdPrd())))
                .orElse(null);
        if (trsAcctInf != null) {
            overdPrd = trsAcctInf.getTrsLateMonth().getOverdPrd();
        }
        return overdPrd;
    }

    /**
     * 贷款信息-当前贷款最大逾期金额
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curMaxOverdAmt(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        BigDecimal totOverd = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        TrsAcctInf trsAcctInf = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getTotOverd() != null)
                .max(Comparator.comparing((e) -> e.getTrsLateMonth().getTotOverd()))
                .orElse(null);
        if (trsAcctInf != null) {
            totOverd = trsAcctInf.getTrsLateMonth().getTotOverd();
        }
        return totOverd;
    }

    /**
     * 贷款信息-当前贷款最大逾期期数对应的最大逾期金额
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curMaxOverdPrdOfAmtCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        BigDecimal totOverd = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        TrsAcctInf trsAcctInf = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getOverdPrd() != null)
                .collect(Collectors.maxBy((e1, e2) -> Integer.compare(e1.getTrsLateMonth().getOverdPrd(), e2.getTrsLateMonth().getOverdPrd())))
                .orElse(null);
        if (trsAcctInf != null) {
            totOverd = trsAcctInf.getTrsLateMonth().getTotOverd();
        }
        return totOverd;
    }

    /**
     * 贷款信息-当前贷款逾期账户数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanOverdActCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        //只统计逾期的
        long count = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth().getAcctStatuCode().equals("2"))
                .count();
        return count;
    }

    /**
     * 贷款信息-贷款信息-当前贷款逾期账户数占比
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanOverdActPropCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        BigDecimal curLoanOverdActProp = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        //只统计逾期的
        long count = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getAcctStatuCode().equals("2"))
                .count();
        if (trsAcctInfs != null && trsAcctInfs.size() > 0) {
            curLoanOverdActProp = new BigDecimal(trsAcctInfs.size()).divide(new BigDecimal(count), 2, BigDecimal.ROUND_CEILING);
        }
        return curLoanOverdActProp;
    }

    /**
     * 贷款信息-当前贷款逾期机构数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanOverdOrgCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<String> magOrgCode = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getAcctStatuCode().equals("2"))
                .map((e) -> {
                    return e.getTrsBase().getMagOrgTypeCode();
                }).collect(Collectors.toList());
        long count = magOrgCode.stream().distinct().count();
        return count;
    }

    /**
     * 贷款信息-当前贷款逾期机构数占比
     * curLoanOverdOrgPropCalculate
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanOverdOrgPropCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<String> magOrgCode = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getAcctStatuCode().equals("2"))
                .map((e) -> {
                    return e.getTrsBase().getMagOrgTypeCode();
                }).collect(Collectors.toList());
        //逾期机构数
        long overdOrgCount = magOrgCode.stream().distinct().count();

        //总机构数
        long orgCount = trsAcctInfs.stream().map((e) -> {
            return e.getTrsBase().getMagOrgTypeCode();
        }).distinct().count();

        BigDecimal overdOrgProp = new BigDecimal("0.00");
        if (overdOrgCount > 0) {
            overdOrgProp = new BigDecimal(overdOrgCount).divide(new BigDecimal(orgCount), 2, BigDecimal.ROUND_CEILING);
        }
        return overdOrgProp;
    }

    /**
     * 贷款信息-近1月贷款的最大逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat1MonMaxOverdPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 1, DateUtils.UNIT_MOUTHS);
        return getMaxOverdPrd(trsAcctInfs, startDate, currDate);
    }

    /**
     * 贷款信息-近3月贷款的最大逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat3MonMaxOverdPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 3, DateUtils.UNIT_MOUTHS);
        return getMaxOverdPrd(trsAcctInfs, startDate, currDate);
    }

    /**
     * 贷款信息-近6月贷款的最大逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat6MonMaxOverdPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 6, DateUtils.UNIT_MOUTHS);
        return getMaxOverdPrd(trsAcctInfs, startDate, currDate);
    }

    /**
     * 贷款信息-近9月贷款的最大逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat9MonMaxOverdPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 9, DateUtils.UNIT_MOUTHS);
        return getMaxOverdPrd(trsAcctInfs, startDate, currDate);
    }

    /**
     * 贷款信息-近24月贷款的最大逾期期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat24MonMaxOverdPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 24, DateUtils.UNIT_MOUTHS);
        return getMaxOverdPrd(trsAcctInfs, startDate, currDate);
    }

    /**
     * 贷款信息-近24月贷款最大逾期距离现在的月数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat24MonMaxOverdMonCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 24, DateUtils.UNIT_MOUTHS);
        //TODO 该算法比较复杂待实现。。。
        AtomicReference<Integer> maxOverdPrd = new AtomicReference<>(0);
        trsAcctInfs.stream()
                .filter((e) -> e.getTrsFiveYear() != null)
                .forEach((e) -> {
                    TrsFiveYear trsFiveYear = e.getTrsFiveYear();
                    //循环近5年内历史表现信息
                    if (trsFiveYear != null) {
                        TrsFiveYearInf trsFiveYearInf = trsFiveYear.getTrsFiveYearInf();
                        if (trsFiveYearInf != null) {
                            List<TrsFiveYearDetail> trsFiveYearDetailList = trsFiveYearInf.getTrsFiveYearDetailList();
                            //按照还款状态代码进行分组
                            List<String> overdStatusCodeList = trsFiveYearDetailList.stream()
                                    .filter((y) -> y.getOverdMon() != null
                                            && DateUtils.convertStrToDate(y.getOverdMon(), DateUtils.YYYY_MM).compareTo(startDate) > 0
                                            && DateUtils.convertStrToDate(y.getOverdMon(), DateUtils.YYYY_MM).compareTo(currDate) < 0
                                    )
                                    .map(TrsFiveYearDetail::getOverdStatusCode)
                                    .collect(Collectors.toList());
                            if (overdStatusCodeList.contains("B") || overdStatusCodeList.contains("G") || overdStatusCodeList.contains("D")) {
                                maxOverdPrd.set(7);
                            } else {
                                String s3 = overdStatusCodeList.stream()
                                        .filter((s) -> s.equals("1") || s.equals("2") || s.equals("3") || s.equals("4") || s.equals("5") || s.equals("6"))
                                        .collect(Collectors.maxBy((s1, s2) -> Integer.valueOf(s1).compareTo(Integer.valueOf(s2))))
                                        .orElse("0");

                                maxOverdPrd.set(Integer.valueOf(s3));
                            }
                        }
                    }
                });
        return maxOverdPrd.get();
    }

    /**
     * 贷款信息-近3月开户最高贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat3MonOpenHigPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenHigPrinCalculate(fieldParamEntities, jsonObject, 3);
    }

    /**
     * 贷款信息-近3月开户最低贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat3MonOpenLowPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenLowPrinCalculate(fieldParamEntities, jsonObject, 3);
    }

    /**
     * 贷款信息-近3月开户平均贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat3MonOpenAvgPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenAvgPrinCalculate(fieldParamEntities, jsonObject, 3);
    }

    /**
     * 贷款信息-近6月开户最高贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat6MonOpenHigPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenHigPrinCalculate(fieldParamEntities, jsonObject, 6);
    }

    /**
     * 贷款信息-近6月开户最低贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat6MonOpenLowPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenLowPrinCalculate(fieldParamEntities, jsonObject, 6);
    }

    /**
     * 贷款信息-近6月开户平均贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat6MonOpenAvgPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenAvgPrinCalculate(fieldParamEntities, jsonObject, 6);
    }

    /**
     * 贷款信息-近12月开户最低贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat12MonOpenLowPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenLowPrinCalculate(fieldParamEntities, jsonObject, 12);
    }

    /**
     * 贷款信息-近12月开户最高贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat12MonOpenHigPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenHigPrinCalculate(fieldParamEntities, jsonObject, 12);
    }

    /**
     * 贷款信息-近12月开户平均贷款本金
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat12MonOpenAvgPrinCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return commonOpenAvgPrinCalculate(fieldParamEntities, jsonObject, 12);
    }

    /**
     * 贷款信息-贷款最近一次还款日期距今时长
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object latRpyDateDaysCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Integer latRpyDateDays = 0;
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        //获取最近一次还款日期最大的值
        TrsAcctInf trsAcctInf = trsAcctInfs.stream()
                .filter(e -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getLatrPyDate() != null)
                .sorted(new Comparator<TrsAcctInf>() {
                    @Override
                    public int compare(TrsAcctInf o1, TrsAcctInf o2) {
                        return o2.getTrsLateMonth().getLatrPyDate().compareTo(o1.getTrsLateMonth().getLatrPyDate());
                    }
                })
                .findFirst()
                .orElse(null);

        if (trsAcctInf != null) {
            Date latrPyDate = trsAcctInf.getTrsLateMonth().getLatrPyDate();
            Date currDate = new Date();
            latRpyDateDays = DateUtils.getSpaceByUnit(latrPyDate, currDate, DateUtils.UNIT_DAYS);
        }
        return latRpyDateDays;
    }

    /**
     * 贷款信息-当前正常贷款账户数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanActCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        Long count = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null
                        && e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_1)
                        && e.getTrsLateMonth().getOverdPrd() == 0)
                .count();
        return count;
    }

    /**
     * 贷款信息-当前未结清贷款账户数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curNoSettleActCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Long curNoSettleActCount = 0L;
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        if (trsAcctInfs != null) {
            curNoSettleActCount = trsAcctInfs.stream().count();
        }
        return curNoSettleActCount;
    }

    /**
     * 贷款信息-当前正常结清贷款账户数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curSettleActCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        Long curSettleActCount = 0L;
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        //统计结清的账号数
        curSettleActCount = trsAcctInfs.stream().filter((e) -> (e.getTrsLateMonth() != null && e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                || (e.getTrsLate() != null && e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_3))
        ).count();
        return curSettleActCount;
    }

    /**
     * 贷款信息-当前正常贷款账户数占比
     * 当前正常贷款账号数/当前未结清贷款账户数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanActCountPropCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        BigDecimal curLoanActCountProp = new BigDecimal("0.00");
        //当前正常贷款账号数
        Object curLoanActCount = curLoanActCountCalculate(fieldParamEntities, fieldDefineEntity, jsonObject);

        //当前未结清贷款账户数
        Object curNoSettleActCount = curNoSettleActCountCalculate(fieldParamEntities, fieldDefineEntity, jsonObject);

        if (curLoanActCount != null) {
            curLoanActCountProp = new BigDecimal((Long) curLoanActCount).divide(new BigDecimal((Long) curNoSettleActCount), 2, BigDecimal.ROUND_CEILING);
        }
        return curLoanActCountProp;
    }

    /**
     * 贷款信息-当前正常结清贷款账户数占比
     * 当前正常结清的贷款账户数/总账户数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curSettleActCountPropCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {

        BigDecimal curSettleActCountProp = new BigDecimal("0.00");
        Long curSettleActCount = 0L;
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        Long totalCount = Long.valueOf(trsAcctInfs.size());
        //统计结清的账号数
        curSettleActCount = trsAcctInfs.stream().filter((e) -> (e.getTrsLateMonth() != null && e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                || (e.getTrsLate() != null && e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_3))
        ).count();

        if (curSettleActCount > 0) {
            curSettleActCountProp = new BigDecimal(curSettleActCount).divide(new BigDecimal(totalCount), 2, BigDecimal.ROUND_CEILING);
        }
        return curSettleActCountProp;
    }

    /**
     * 贷款信息-当前未结清贷款余额总和
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curNoSettleBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        BigDecimal busiBalSum = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null)
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return busiBalSum;
    }

    /**
     * 贷款信息-当前未结清住房贷款余额总和
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curNoSettleHousBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = loanTypeTrsAccts(fieldParamEntities, jsonObject, Arrays.asList("11", "12", "13"));
        //找到未结清的信息
        BigDecimal noSettleHousBal = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return noSettleHousBal;
    }

    /**
     * 贷款信息-当前未结清汽车贷款余额总和
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curNoSettleCarBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = loanTypeTrsAccts(fieldParamEntities, jsonObject, Arrays.asList("21"));
        //找到未结清的信息
        BigDecimal noSettleHousBal = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return noSettleHousBal;
    }

    /**
     * 贷款信息-当前未结清个人经营性贷款余额总和
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curNoSettleBusBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = loanTypeTrsAccts(fieldParamEntities, jsonObject, Arrays.asList("41"));
        //找到未结清的信息
        BigDecimal noSettleHousBal = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return noSettleHousBal;
    }

    /**
     * 贷款信息-当前平均每月贷款余额总和
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curAvgMonBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {

        BigDecimal curAvgMonBalSum = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);

        BigDecimal noSettleHousBal = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null)
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        if (noSettleHousBal != null && noSettleHousBal.compareTo(BigDecimal.ZERO) > 0) {
            curAvgMonBalSum = noSettleHousBal.divide(new BigDecimal("12"), 2, BigDecimal.ROUND_CEILING);
        }
        return curAvgMonBalSum;
    }

    /**
     * 贷款信息-当前正常贷款账户余额
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanActBalSumCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        BigDecimal curLoanActBalSum = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getOverdPrd() == 0)
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                }).reduce(BigDecimal.ZERO, BigDecimal::add);
        return curLoanActBalSum;
    }

    /**
     * 贷款信息-当前正常贷款账户余额占总余额比
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curLoanActBalPropCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        BigDecimal curLoanActBalProp = new BigDecimal("0.00");

        //当前正常贷款账户余额
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        BigDecimal curLoanActBalSum = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getOverdPrd() == 0)
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                }).reduce(BigDecimal.ZERO, BigDecimal::add);

        //未结清贷款余额总额
        BigDecimal busiBalSum = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null)
                .map((e) -> {
                    TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                    return trsLateMonth.getBusiBal();
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        if (curLoanActBalSum != null && curLoanActBalSum.compareTo(BigDecimal.ZERO) > 0) {
            curLoanActBalProp = curLoanActBalSum.divide(busiBalSum, 2, BigDecimal.ROUND_CEILING);
        }
        return curLoanActBalProp;
    }

    /**
     * 贷款信息-贷款24期最大还款记录次数
     * TODO--待实现
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object loan24PrdMaxRpyCountCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        return 0;
    }

    /**
     * 贷款信息-贷款本月实还款金额
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object loanActRpyAmtCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        BigDecimal loanActRpyAm = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null && e.getTrsLateMonth().getActRpyAmt() != null)
                .map((e) ->
                        {
                            TrsLateMonth trsLateMonth = e.getTrsLateMonth();
                            return trsLateMonth.getActRpyAmt();
                        }
                )
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return loanActRpyAm;
    }

    /**
     * 贷款信息-最近3个月个人消费贷款发放额度
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat3MPerConsLimitCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = loanTypeTrsAccts(fieldParamEntities, jsonObject, Arrays.asList("21", "41", "53", "91"));

        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 3, DateUtils.UNIT_MOUTHS);
        BigDecimal lat3MPerConsLimit = trsAcctInfs.stream()
                .filter((e) -> e.getTrsBase() != null)
                .filter((e) -> DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(startDate) > 0
                        && DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(currDate) < 0)
                .map((e) -> e.getTrsBase().getLoanAmt())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return lat3MPerConsLimit;
    }

    /**
     * 贷款信息-最近6个月个人消费贷款发放额度
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat6MPerConsLimitCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = loanTypeTrsAccts(fieldParamEntities, jsonObject, Arrays.asList("21", "41", "53", "91"));

        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 6, DateUtils.UNIT_MOUTHS);
        BigDecimal lat6MPerConsLimit = trsAcctInfs.stream()
                .filter((e) -> e.getTrsBase() != null)
                .filter((e) -> DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(startDate) > 0
                        && DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(currDate) < 0)
                .map((e) -> e.getTrsBase().getLoanAmt())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return lat6MPerConsLimit;
    }

    /**
     * 贷款信息-最近12个月个人消费贷款发放额度
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object lat12MPerConsLimitCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = loanTypeTrsAccts(fieldParamEntities, jsonObject, Arrays.asList("21", "41", "53", "91"));
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, 12, DateUtils.UNIT_MOUTHS);
        BigDecimal lat12MPerConsLimit = trsAcctInfs.stream()
                .filter((e) -> e.getTrsBase() != null)
                .filter((e) -> DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(startDate) > 0
                        && DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(currDate) < 0)
                .map((e) -> e.getTrsBase().getLoanAmt())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return lat12MPerConsLimit;
    }

    /**
     * 贷款信息-未结清贷款平均剩余还款期数
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object noSettleAvgRemRepPrdCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {

        BigDecimal noSettleAvgRemRepPrd = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanNoSettleAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> trsAcctInfList = trsAcctInfs.stream()
                .filter((e) -> e.getTrsLateMonth() != null)
                .collect(Collectors.toList());
        int acctCount = trsAcctInfList.size();
        Integer remRepPrdSum = trsAcctInfList.stream()
                .collect(Collectors.summingInt((e) -> Optional.ofNullable(e.getTrsLateMonth().getRemRepPrd()).orElse(0)));
        if (remRepPrdSum > 0) {
            noSettleAvgRemRepPrd = new BigDecimal(remRepPrdSum).divide(new BigDecimal(acctCount), 2, BigDecimal.ROUND_CEILING);
        }
        return noSettleAvgRemRepPrd;
    }

    /**
     * 贷款信息-当前贷款本月应还金额总和
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object curRpyAmtCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfList = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        BigDecimal curRpyAmtSum = trsAcctInfList.stream()
                .filter((e) -> e.getTrsLateMonth() != null)
                .map((e) -> e.getTrsLateMonth().getCurrPyAmt())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return curRpyAmtSum;
    }

    /**
     * 贷款信息-当前贷款本月实还金额总额
     *
     * @param fieldParamEntities
     * @param fieldDefineEntity
     * @param jsonObject
     * @return
     */
    public Object actRpyAmtCalculate(List<FieldParamEntity> fieldParamEntities, FieldDefineEntity fieldDefineEntity, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfList = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        BigDecimal actRpyAmt = trsAcctInfList.stream()
                .filter((e) -> e.getTrsLateMonth() != null)
                .map((e) -> e.getTrsLateMonth().getActRpyAmt())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        return actRpyAmt;
    }

    /**
     * 获取到 业务种类相关的账号信息list
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @param loanTypeCodeList
     */
    public List<TrsAcctInf> loanTypeTrsAccts(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject, List loanTypeCodeList) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> trsAcctInfList = trsAcctInfs.stream()
                .filter((e) ->
                        (e.getTrsLate() != null && (!e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_52)))
                                || (e.getTrsLateMonth() != null && (!e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_51) || !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_52)))
                )
                .filter((e) -> {
                            if (loanTypeCodeList.contains(e.getTrsBase().getLoanTypeCode())) {
                                return true;
                            } else {
                                return false;
                            }
                        }
                )
                .collect(Collectors.toList());
        return trsAcctInfList;
    }

    /**
     * 抽取公共的最高本金的方法
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @param mounths            月份
     */
    private BigDecimal commonOpenHigPrinCalculate(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject, int mounths) {
        BigDecimal lat3MonOpenHigPrin = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, mounths, DateUtils.UNIT_MOUTHS);

        TrsAcctInf trsAcctInf = trsAcctInfs.stream()
                .filter((e) -> DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(startDate) > 0
                        && DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(currDate) < 0)
                .sorted(new Comparator<TrsAcctInf>() {
                    @Override
                    public int compare(TrsAcctInf o1, TrsAcctInf o2) {
                        return o1.getTrsBase().getLoanAmt().compareTo(o2.getTrsBase().getLoanAmt());
                    }
                })
                .findFirst()
                .orElse(null);
        if (trsAcctInf != null) {
            lat3MonOpenHigPrin = trsAcctInf.getTrsBase().getLoanAmt();
        }
        return lat3MonOpenHigPrin;
    }

    /**
     * 抽取公共的平均本金的方法
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @param mounths
     * @return
     */
    private BigDecimal commonOpenAvgPrinCalculate(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject, int mounths) {
        BigDecimal latMonOpenAvgPrin = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, mounths, DateUtils.UNIT_MOUTHS);
        BigDecimal sumLoanAmt = trsAcctInfs.stream()
                .filter((e) -> DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(startDate) > 0
                        && DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(currDate) < 0)
                .map(trsAcctInf -> {
                    TrsBase trsBase = trsAcctInf.getTrsBase();
                    if (trsBase != null) {
                        return trsBase.getLoanAmt();
                    } else {
                        return new BigDecimal("0.00");
                    }
                })
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        int size = trsAcctInfs.size();
        if (sumLoanAmt.compareTo(BigDecimal.ZERO) > 0) {
            latMonOpenAvgPrin = sumLoanAmt.divide(new BigDecimal(size), 2, BigDecimal.ROUND_CEILING);
        }
        return latMonOpenAvgPrin;
    }

    /**
     * 抽取公共的最低贷款本金的方法
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @param mounths            月份
     */
    private BigDecimal commonOpenLowPrinCalculate(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject, int mounths) {
        BigDecimal lat3MonOpenHigPrin = new BigDecimal("0.00");
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        //当前日期
        Date currDate = new Date();
        Date startDate = DateUtils.lastMonthDate(currDate, mounths, DateUtils.UNIT_MOUTHS);

        TrsAcctInf trsAcctInf = trsAcctInfs.stream()
                .filter((e) -> DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(startDate) > 0
                        && DateUtils.convertStrToDate(e.getTrsBase().getOpenDate(), DateUtils.YYYY_MM_DD).compareTo(currDate) < 0)
                .sorted(new Comparator<TrsAcctInf>() {
                    @Override
                    public int compare(TrsAcctInf o1, TrsAcctInf o2) {
                        return o2.getTrsBase().getLoanAmt().compareTo(o1.getTrsBase().getLoanAmt());
                    }
                })
                .findFirst()
                .orElse(null);
        if (trsAcctInf != null) {
            lat3MonOpenHigPrin = trsAcctInf.getTrsBase().getLoanAmt();
        }
        return lat3MonOpenHigPrin;
    }

    /**
     * 抽取公共的获取最大逾期期数的算法
     *
     * @param trsAcctInfs
     * @param startDate
     * @param currDate
     * @return
     */
    private Integer getMaxOverdPrd(List<TrsAcctInf> trsAcctInfs, Date startDate, Date currDate) {
        AtomicReference<Integer> maxOverdPrd = new AtomicReference<>(0);
        trsAcctInfs.stream()
                .filter((e) -> e.getTrsFiveYear() != null)
                .forEach((e) -> {
                    TrsFiveYear trsFiveYear = e.getTrsFiveYear();
                    //循环近5年内历史表现信息
                    if (trsFiveYear != null) {
                        TrsFiveYearInf trsFiveYearInf = trsFiveYear.getTrsFiveYearInf();
                        if (trsFiveYearInf != null) {
                            List<TrsFiveYearDetail> trsFiveYearDetailList = trsFiveYearInf.getTrsFiveYearDetailList();
                            //按照还款状态代码进行分组
                            List<String> overdStatusCodeList = trsFiveYearDetailList.stream()
                                    .filter((y) -> y.getOverdMon() != null
                                            && DateUtils.convertStrToDate(y.getOverdMon(), DateUtils.YYYY_MM).compareTo(startDate) > 0
                                            && DateUtils.convertStrToDate(y.getOverdMon(), DateUtils.YYYY_MM).compareTo(currDate) < 0
                                    )
                                    .map(TrsFiveYearDetail::getOverdStatusCode)
                                    .collect(Collectors.toList());
                            if (overdStatusCodeList.contains("B") || overdStatusCodeList.contains("G") || overdStatusCodeList.contains("D")) {
                                maxOverdPrd.set(7);
                            } else {
                                String s3 = overdStatusCodeList.stream()
                                        .filter((s) -> s.equals("1") || s.equals("2") || s.equals("3") || s.equals("4") || s.equals("5") || s.equals("6"))
                                        .collect(Collectors.maxBy((s1, s2) -> Integer.valueOf(s1).compareTo(Integer.valueOf(s2))))
                                        .orElse("0");

                                maxOverdPrd.set(Integer.valueOf(s3));
                            }
                        }
                    }
                });
        return maxOverdPrd.get();
    }

    /**
     * 是否包含关键字
     *
     * @param magOrgCodeList
     * @param type
     * @return
     */
    private Boolean isContain(List<String> magOrgCodeList, String type) {
        Boolean containFlag = false;
        for (String str : magOrgCodeList) {
            if (type.contains(str)) {
                containFlag = true;
                break;
            }
        }
        return false;
    }

    /**
     * 合并贷款信息记录-包含已结清的
     *
     * @param fieldParamEntities
     * @param jsonObject
     */
    private List<TrsAcctInf> mergeLoanAcctTrsInf(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject) {
        Map<String, List<FieldParamEntity>> mapFieldParam = fieldParamEntities.stream()
                .filter((e) -> e.getParamValue() != null && !e.getParamValue().equals(""))
                .collect(Collectors.groupingBy((e) -> e.getParamValue()));
        List<TrsAcctInf> acctTrsInfs = new ArrayList<>();
        mapFieldParam.forEach((k, v) -> {
            IXmlBase jsonBean = getXmlBase(k, jsonObject);
            if (jsonBean != null) {
                //循环贷账户
                if (jsonBean instanceof LoopAcctTrsInf) {
                    LoopAcctTrsInf loopAcctTrsInf = (LoopAcctTrsInf) jsonBean;
                    List<TrsAcctInf> trsAcctInfList = loopAcctTrsInf.getTrsAcctInfList();
                    if (trsAcctInfList != null) {
                        List<TrsAcctInf> loopAcctTrsInfList = trsAcctInfList.stream()
                                .filter((e) -> e.getTrsBase().getBusiOrgCode() != null)
                                .collect(Collectors.toList());
                        acctTrsInfs.addAll(loopAcctTrsInfList);
                    }
                }
                //循环额度下分账户
                if (jsonBean instanceof LoopQuotaAcctTrsInf) {
                    LoopQuotaAcctTrsInf loopQuotaAcctTrsInf = (LoopQuotaAcctTrsInf) jsonBean;
                    List<TrsAcctInf> trsAcctInfList = loopQuotaAcctTrsInf.getTrsAcctInfList();
                    if (trsAcctInfList != null) {
                        List<TrsAcctInf> loopQuotaAcctTrsInfList = trsAcctInfList.stream()
                                .filter((e) -> e.getTrsBase().getBusiOrgCode() != null)
                                .collect(Collectors.toList());
                        acctTrsInfs.addAll(loopQuotaAcctTrsInfList);
                    }
                }
                //非循环贷账户
                if (jsonBean instanceof NoLoopAcctTrsInf) {
                    NoLoopAcctTrsInf noLoopAcctTrsInf = (NoLoopAcctTrsInf) jsonBean;
                    List<TrsAcctInf> trsAcctInfList = noLoopAcctTrsInf.getTrsAcctInfList();
                    if (trsAcctInfList != null) {
                        List<TrsAcctInf> noLoopAcctTrsInfList = trsAcctInfList.stream()
                                .filter((e) -> e.getTrsBase().getBusiOrgCode() != null)
                                .collect(Collectors.toList());
                        acctTrsInfs.addAll(noLoopAcctTrsInfList);
                    }
                }
            }
        });
        return acctTrsInfs;
    }

    /**
     * 合并贷款信息记录 非结清的记录
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @return
     */
    private List<TrsAcctInf> mergeLoanNoSettleAcctTrsInf(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject) {
        List<TrsAcctInf> trsAcctInfs = mergeLoanAcctTrsInf(fieldParamEntities, jsonObject);
        List<TrsAcctInf> noSettleAcctTrsInf = trsAcctInfs.stream()
                .filter((e) -> (e.getTrsLateMonth() != null && !e.getTrsLateMonth().getAcctStatuCode().equals(BaseConsts.ACCT_STATUS_3))
                        || (e.getTrsLate() != null && !e.getTrsLate().getAcctStatusCode().equals(BaseConsts.ACCT_STATUS_3))
                )
                .collect(Collectors.toList());
        return noSettleAcctTrsInf;
    }

    /**
     * 获取值指定字段的值
     *
     * @param object
     * @param field
     * @return
     */
    private Object getValue(Object object, String field) {
        Class<?> aClass = object.getClass();
        Field declaredField = null;
        Object value = null;
        try {
            declaredField = aClass.getDeclaredField(field);
            declaredField.setAccessible(true);
            value = declaredField.get(object);
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
        }
        return value;
    }

    /**
     * 抽取公共的算法
     *
     * @param fieldParamEntities
     * @param jsonObject
     * @param currOverPrdAmtSum
     * @return
     */
    private void commonCalculate(List<FieldParamEntity> fieldParamEntities, JSONObject jsonObject, AtomicReference<BigDecimal> currOverPrdAmtSum) {
        fieldParamEntities.stream().forEach((e) -> {
            if (e.getParamType().equals("xml")) {
                String paramValue = e.getParamValue();
                IXmlBase jsonBean = getXmlBase(paramValue, jsonObject);
                if (jsonBean != null) {
                    //循环贷账户
                    if (jsonBean instanceof LoopAcctTrsInf) {
                        LoopAcctTrsInf loopAcctTrsInf = (LoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopAcctTrsInf.getTrsAcctInfList();
                        BigDecimal loopAcctTotOverd = getBigDecimal(trsAcctInfList, e.getFieldName());
                        currOverPrdAmtSum.set(currOverPrdAmtSum.get().add(loopAcctTotOverd));
                    }
                    //循环额度下分账户
                    if (jsonBean instanceof LoopQuotaAcctTrsInf) {
                        LoopQuotaAcctTrsInf loopQuotaAcctTrsInf = (LoopQuotaAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = loopQuotaAcctTrsInf.getTrsAcctInfList();
                        BigDecimal loopQuotaAcctTotOverd = getBigDecimal(trsAcctInfList, e.getFieldName());
                        currOverPrdAmtSum.set(currOverPrdAmtSum.get().add(loopQuotaAcctTotOverd));
                    }
                    //非循环贷账户
                    if (jsonBean instanceof NoLoopAcctTrsInf) {
                        NoLoopAcctTrsInf noLoopAcctTrsInf = (NoLoopAcctTrsInf) jsonBean;
                        List<TrsAcctInf> trsAcctInfList = noLoopAcctTrsInf.getTrsAcctInfList();
                        BigDecimal noLoopAcctTotOverd = getBigDecimal(trsAcctInfList, e.getFieldName());
                        currOverPrdAmtSum.set(currOverPrdAmtSum.get().add(noLoopAcctTotOverd));
                    }
                }
            }
        });
    }

    /**
     * 获取金额的之和通用方法
     *
     * @param trsAcctInfList
     * @param fieldName
     * @return
     */
    private BigDecimal getBigDecimal(List<TrsAcctInf> trsAcctInfList, String fieldName) {
        BigDecimal totOverd = new BigDecimal("0.00");
        if (trsAcctInfList != null) {
            totOverd = trsAcctInfList.stream()
                    .filter((trsAcctInf) -> trsAcctInf.getTrsLateMonth() != null && trsAcctInf.getTrsLateMonth().getTotOverd() != null)
                    .map((trsAcctInf) -> {
                        TrsLateMonth trsLateMonth = trsAcctInf.getTrsLateMonth();
                        BigDecimal value = (BigDecimal) getValue(trsLateMonth, fieldName);
                        return value;
                    })
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        return totOverd;
    }


}
