package com.kexun.service.impl;

import com.kexun.entity.ReportFileEntity;
import com.kexun.entity.ReportInfo;
import com.kexun.service.ReportFileService;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Description: 文件处理实现类，包括文件的读取、写入
 *
 * @author lgs
 * @date 2023/12/22
 */
@Service
public class FileServiceImpl {

    @Autowired
    ReportFileService reportFileService;
    /**
     * xml文件来源路径
     */
    @Value("${report.file.xmlSourcePath}")
    private String xmlSourcePath;
    /**
     * 解析出来的json文件
     */
    @Value("${report.file.jsonTargetPath}")
    private String jsonTargetPath;
    /**
     * 解析出来的json文件
     */
    @Value("${report.file.xmlTargetPath}")
    private String xmlTargetPath;

    /**
     * 上传文件
     * @param context
     * @param filename
     * @param path
     */
    private void writeFile(String context, String filename,String path) {
        try {
            IOUtils.write(context, new FileOutputStream(path + filename), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 保存xml文件和json文件
     * @param reportInfo
     * @param xmlContext
     * @param jsonContext
     * @param fileName
     */
    public void saveAndWriteFile(ReportInfo reportInfo,String xmlContext,String jsonContext,String fileName){
        ReportFileEntity reportFileEntity = new ReportFileEntity();
        reportFileEntity.setHeadInfId(reportInfo.getHeadInfId());
        reportFileEntity.setRptNo(reportInfo.getRptNo());
        reportFileEntity.setBusinessId(reportInfo.getBusinessId());
        reportFileEntity.setBusinessNum(reportInfo.getBusinessNum());
        reportFileEntity.setFileNameXml(fileName+".xml");
        reportFileEntity.setFileNameTxt(fileName+".txt");
        //保存文件信息表
        reportFileService.saveOrUpdate(reportFileEntity);
        //上传文件到服务器的目录
        //xml文件
        writeFile(xmlContext,fileName+".xml",xmlTargetPath);
        //json文件
        writeFile(jsonContext,fileName+".txt",jsonTargetPath);
    }

}
